import ij.*;
import ij.process.*;
import ij.gui.*;
/**
 * Subtract 32bit-REAL Stack and create a new Stack or a new Image.
 * @author Koutaro Abe ( kabe@cris.hokudai.ac.jp )
 * @version 07-Jun-2010
 */
public class Delta {

	public ImagePlus imp;
	private int zD, zS, size, div;
	private boolean fixFront;
	private ImageStack src, result;
	private static String newTitle;
	private static String cmdEach = "each";
	private static String cmdAll = "total";
	private static String cmdAllDiv = "total+div";
	private static String cmdIntegral = "integral";
	
	/**
	 * @param imp
	 *   ImagePlus including source stack.
	 * @param command
	 * 	 'each': difference between each slice and in front of it.
	 *   'total': difference between bottom slice and top slice.
	 *   'total+div': and divide each pixel by number of slices.
	 *   'integral': difference between each slice and top slice.
	 *   otherwise : show dialog.
	 * @param zDelta
	 *   depth between two slices. ( 0<zDelta )
	 * @param zStep
	 *   increase value of slice counter. ( 0<zStep )
	 */
	public Delta( ImagePlus imageplus, String command, int zDelta, int zStep ) {
		imp = imageplus;
		if( imp==null) {
			IJ.showMessage( "Delta: Please select a Stack Image window.");
			return;
		}
		src = imp.getStack();
		if( src!=null && ( ( size=src.getSize() ) > 1) ) {
			if( command.equals( cmdEach ) ) {
				zD = 1; zS = 1;
				fixFront = false; div = 1;
				newTitle = "Dd-" + imp.getTitle();
			} else if ( command.equals( cmdAll ) ) {
				zD = size-1; zS = size;
				fixFront = false; div = 1;
				newTitle = "Dt-" + imp.getTitle();
			} else if ( command.equals( cmdAllDiv ) ) {
				zD = size-1; zS = size;
				fixFront = false; div = size - 1;
				newTitle = "Dt" + div + "-" + imp.getTitle(); 
			} else if ( command.equals( cmdIntegral ) ) {
				zD = 1; zS = 1;
				fixFront = true; div = 1;
				newTitle = "Di-" + imp.getTitle();
			} else {
				zD = zDelta; zS = zStep;
				fixFront = false; div = 1;
				newTitle = "D-" + imp.getTitle();
				if( zD<1 || zS<1 )
					dialog();
			}
		} else {
			IJ.showMessage( "Delta: Selected Image is not a Stack.");
		}
		result = getStack();
	}
	public Delta( ImagePlus imp, String cmd ) {	this(imp, cmd, 0, 0 ); }
	public Delta( ImagePlus imp ) { this(imp, null, 0, 0 ); }
	public ImagePlus getImp() { return imp; }
	public String getTitle() { return newTitle; }
	
	private boolean dialog() {
		GenericDialog gd = new GenericDialog( "Delta - Parameters" );
		gd.addNumericField( "Delta between :", 1, 0, 6, " slice(s)" );
		gd.addNumericField( "Interval:", 1, 0, 6, " slice(s)" );
		gd.addCheckbox( "Integral", fixFront );	
		gd.addNumericField( "divide by :", size-1, 0, 6, "" );
		gd.showDialog();
		
		if( gd.wasCanceled() ) return false;
		zD = (int) gd.getNextNumber();
		zS = (int) gd.getNextNumber();
		fixFront = gd.getNextBoolean();
		div = (int) gd.getNextNumber();
		if( div==0 ) div=1;
		return true;
	}

	public ImageStack getStack( ) {
		if( src==null || zD==0 || zS==0 || div==0 ) return null;	
		float[] front, back, result; 
		String frontName, backName;
		int width = src.getWidth();
		int height = src.getHeight();
		int pixels = width * height;	

		IJ.showStatus( "Delta process status" );
		IJ.showProgress( 0.0 );
		ImageStack resultStack = new ImageStack( width, height );
		for( int i = 1; i <= size - zD; i = i + zS  ) {
			IJ.showProgress(  i, size-zD );
			back  = (float[]) src.getPixels( i + zD );
			backName = src.getSliceLabel( i + zD );
			int frontSlice = fixFront ? 1 : i;
			front = (float[]) src.getPixels( frontSlice );
			frontName = src.getSliceLabel( frontSlice );
			ImageProcessor result_ip = new FloatProcessor( width, height );
 			result = (float[]) result_ip.getPixels();
			for( int j = 0; j < pixels; j++ ) {
				result[j] =  ( back[j] - front[j] ) / div;
			}
			resultStack.addSlice( frontName + " - " + backName, result_ip );
		}
		return resultStack;
	}
	
	public void show( String title ){
		newTitle = title;
		if( result==null ) return;
		ImagePlus window = new ImagePlus( title, result );
		if ( window != null ) window.show();
	}

	public void show(){
		show( newTitle );
	}
}
